/*******************************************************************
 *
 *	File:		ArloApp.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <eikenv.h>
#include <eikdialg.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <eikdlgtb.h>
#include <eiklabel.h>
#include <eikchlst.h>
#include <eikchkbx.h>
#include <eikdialg.hrh>
#include <eikctgrp.h>
#include <eikfsel.h>
#include <eikcfdlg.h>
#include <eikmfne.h>
#include <eikon.rsg>

#include "ArloApp.h"
#include "Arlo.rsg"
#include "Arlo.mbg"
#include "version.h"

#include "CArloConfig.h"

#define ONE_MB (1024 * 1024)


/************************************************
 *
 * Arlo information dialog
 *
 ************************************************/

class CArloInfoDialog : public CEikDialog
{
public:
	CArloInfoDialog(TUint aProcId) : iProcId(aProcId){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
	TUint	iProcId;
};

void CArloInfoDialog::PreLayoutDynInitL()
{
	TBuf<128>	text;

	TMachineInfoV1Buf		machineInfo;
	TBuf<64>				machineId;

	if (UserHal::MachineInfo(machineInfo) == KErrNone)
	{
		machineId.Num(machineInfo().iMachineUniqueId);
		text.Format(_L("%S, Id: %S"), &machineInfo().iMachineName, &machineId);
		CEikLabel*	label =(CEikLabel*)Control(ECtlArloInfoMachine);
		label->SetTextL(text);

		text.Format(_L("%S (%d MHz), Speedfactor %d, id 0x%x"), &machineInfo().iProcessorName, (machineInfo().iProcessorClockInKHz / 1024), machineInfo().iSpeedFactor, iProcId);
		label =(CEikLabel*)Control(ECtlArloInfoProc);
		label->SetTextL(text);

		label =(CEikLabel*)Control(ECtlArloInfoRomVersion);
		label->SetTextL(machineInfo().iRomVersion.Name());
	}
	else
	{
		CEikLabel*	label =(CEikLabel*)Control(ECtlArloInfoMachine);
		label->SetTextL(_L("Unknown"));
		label =(CEikLabel*)Control(ECtlArloInfoProc);
		label->SetTextL(_L("Unknown"));
		label =(CEikLabel*)Control(ECtlArloInfoRomVersion);
		label->SetTextL(_L("Unknown"));
	}

	TMemoryInfoV1Buf		memoryInfo;
	if (UserHal::MemoryInfo(memoryInfo) == KErrNone)
	{
		text.Format(_L("%d MB (%d MB Ramdisk)"), memoryInfo().iTotalRamInBytes / ONE_MB, memoryInfo().iInternalDiskRamInBytes / ONE_MB);
		CEikLabel*	label =(CEikLabel*)Control(ECtlArloInfoRam);
		label->SetTextL(text);

		TBuf<8>	reprog = memoryInfo().iRomIsReprogrammable ? _L("is") : _L("is not");
		text.Format(_L("%d MB Rom, Rom %S reprogrammable"), memoryInfo().iTotalRomInBytes / ONE_MB, &reprog);
		label =(CEikLabel*)Control(ECtlArloInfoRom);
		label->SetTextL(text);
	}
	else
	{
		CEikLabel*	label =(CEikLabel*)Control(ECtlArloInfoRam);
		label->SetTextL(_L("Unknown"));
		label =(CEikLabel*)Control(ECtlArloInfoRom);
		label->SetTextL(_L("Unknown"));
	}
}

void CArloInfoDialog::SetSizeAndPositionL(const TSize& aSize)
{
//	TSize	size(400, 200);
	SetCornerAndSizeL(EHCenterVCenter, aSize);
}



/************************************************
 *
 * Arlo options dialog
 *
 ************************************************/

class CArloOptionsDialog : public CEikDialog
{
public:
	~CArloOptionsDialog();
	CArloOptionsDialog(CArloConfig& aArloConfig) : iArloConfig(aArloConfig){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
	void HandleControlStateChangeL(TInt aControlId);
	TBool OkToExitL(TInt aKeycode);

	void SetUiState();
	void GetData();

protected:
	CDesCArrayFlat*		iBootItemList;
	CEikChoiceList*		iDefaultBootItemChoiceList;
	CEikNumberEditor*	iTimeoutNumEdit;
	CEikCheckBox*		iAutoboot;

	CArloConfig&		iArloConfig;
};


CArloOptionsDialog::~CArloOptionsDialog()
{
	delete iBootItemList;
}

void CArloOptionsDialog::PreLayoutDynInitL()
{
	iBootItemList = new(ELeave) CDesCArrayFlat(5);
	for (TInt i=0 ; i<iArloConfig.BootItems() ; i++)
		iBootItemList->AppendL(iArloConfig.BootItem(i).iDescription);
	iBootItemList->AppendL(_L("<none>"));
	iDefaultBootItemChoiceList = (CEikChoiceList*) Control(ECtlArloOptionsDefaultBootItem);
	iDefaultBootItemChoiceList->SetArrayL(iBootItemList);
	iDefaultBootItemChoiceList->SetArrayExternalOwnership(ETrue);
	TInt select = (iArloConfig.DefaultBootItem() == -1) ? iArloConfig.BootItems() : iArloConfig.DefaultBootItem();
	iDefaultBootItemChoiceList->SetCurrentItem(select);

	TInt timeout = iArloConfig.Timeout();
	iTimeoutNumEdit = (CEikNumberEditor*)Control(ECtlArloOptionsTimeout);
	iTimeoutNumEdit->SetNumber((timeout == -1) ? 1 : timeout);

	iAutoboot = (CEikCheckBox*)Control(ECtlArloOptionsAutoboot);
	iAutoboot->SetState((timeout!=-1) ? CEikButtonBase::ESet : CEikButtonBase::EClear);
	
	SetUiState();
}

void CArloOptionsDialog::GetData()
{
	TInt defaultBootItem = iDefaultBootItemChoiceList->CurrentItem();
	if (defaultBootItem == iArloConfig.BootItems())
		defaultBootItem = -1;
	iArloConfig.SetDefaultBootItem(defaultBootItem);

	TInt	timeout = -1;
	if (defaultBootItem != -1)
	{
		if (iAutoboot->State() == CEikButtonBase::ESet)
			timeout = iTimeoutNumEdit->Number();
	}
	iArloConfig.SetTimeout(timeout);
}

void CArloOptionsDialog::SetUiState()
{
	TBool hasDefaultboot = iArloConfig.DefaultBootItem() != -1;
	MakeLineVisible(ECtlArloOptionsAutoboot, hasDefaultboot);
	TBool hasTimeout = (hasDefaultboot) ? iArloConfig.Timeout() != -1 : EFalse;
	MakeLineVisible(ECtlArloOptionsTimeout, hasTimeout);
}


void CArloOptionsDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize size = aSize;
	if (size.iWidth < 400)
		size.iWidth = 400;
	SetCornerAndSizeL(EHCenterVCenter, size);
}

void CArloOptionsDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case ECtlArloOptionsDefaultBootItem:
		case ECtlArloOptionsAutoboot:
			GetData();
			SetUiState();
			break;
	}
}

TBool CArloOptionsDialog::OkToExitL(TInt aKeycode)
{
	GetData();
	return ETrue;
}



/************************************************
 *
 * New or edit boot item dialog
 *
 ************************************************/

class CNewOrEditBootItemDialog : public CEikDialog
{
public:
	~CNewOrEditBootItemDialog();
	CNewOrEditBootItemDialog(const CArloConfig&	aArloConfig, TArloBootItem& aArloBootItem, TBool aNewItem) : 
		iArloConfig(aArloConfig), iArloBootItem(aArloBootItem), iNewItem(aNewItem) {}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
	void HandleControlStateChangeL(TInt aControlId);
	TBool OkToExitL(TInt aKeycode);

	void SetInitrdFileName();
	void SetKernelFileName();
	TBool SelectFileL(TDes* aFileName, TInt aTitleId);

protected:
	TArloBootItem&		iArloBootItem;
	CDesCArrayFlat*		iKernelList;
	CDesCArrayFlat*		iInitrdList;
	CEikChoiceList*		ikernelChoiceList;
	CEikChoiceList*		iInitrdChoiceList;
	CEikEdwin*			iDescriptionEdit;
	CEikEdwin*			iBootStringEdit;

	TBool				iNewItem;
	TBool				iIsEpocBootItem;
	const CArloConfig&	iArloConfig;
};


CNewOrEditBootItemDialog::~CNewOrEditBootItemDialog()
{
	delete iKernelList;
	delete iInitrdList;
}


TBool CNewOrEditBootItemDialog::OkToExitL(TInt aKeycode)
{
	TBool	allowed = ETrue;
	if ((aKeycode == EEikBidOk) && !iIsEpocBootItem)
	{
		iDescriptionEdit->GetText(iArloBootItem.iDescription);
		iBootStringEdit->GetText(iArloBootItem.iBootString);

		if (iArloBootItem.iDescription.Length() == 0)
		{
			User::InfoPrint(_L("Please enter a description"));
			allowed = EFalse;
		}
		else if (iNewItem && iArloConfig.HasDescription(iArloBootItem.iDescription))
		{
			User::InfoPrint(_L("Description already exists"));
			allowed = EFalse;
		}
		else if (iArloBootItem.iKernelFileName.Length() == 0)
		{
			User::InfoPrint(_L("Please select a kernel image"));
			allowed = EFalse;
		} 
		else if (iInitrdChoiceList->CurrentItem() == 2)
			iArloBootItem.iInitrdFileName = _L("");
	}

    return allowed;
}


void CNewOrEditBootItemDialog::SetKernelFileName()
{
	iKernelList->Reset();
	if (iArloBootItem.iKernelFileName.Length() == 0)
		iKernelList->AppendL(_L("<none>"));
	else
		iKernelList->AppendL(iArloBootItem.iKernelFileName);
	iKernelList->AppendL(_L("Select file"));
}


void CNewOrEditBootItemDialog::SetInitrdFileName()
{
	iInitrdList->Reset();
	if (iArloBootItem.iInitrdFileName.Length() == 0)
		iInitrdList->AppendL(_L("<none>"));
	else
		iInitrdList->AppendL(iArloBootItem.iInitrdFileName);
	iInitrdList->AppendL(_L("Select file"));
	iInitrdList->AppendL(_L("No initrd"));
}

void CNewOrEditBootItemDialog::PreLayoutDynInitL()
{
	iKernelList = new(ELeave) CDesCArrayFlat(5);
	ikernelChoiceList = (CEikChoiceList*) Control(ECtlNewOrEditBootItemKernel);
	ikernelChoiceList->SetArrayL(iKernelList);
	ikernelChoiceList->SetArrayExternalOwnership(ETrue);
	SetKernelFileName();

	iInitrdList = new(ELeave) CDesCArrayFlat(5);
	iInitrdChoiceList = (CEikChoiceList*) Control(ECtlNewOrEditBootItemInitrd);
	iInitrdChoiceList->SetArrayExternalOwnership(ETrue);
	iInitrdChoiceList->SetArrayL(iInitrdList);
	SetInitrdFileName();

	iDescriptionEdit = (CEikEdwin*)Control(ECtlNewOrEditBootItemDescription);
	iDescriptionEdit->SetTextL(&iArloBootItem.iDescription);

	iBootStringEdit = (CEikEdwin*)Control(ECtlNewOrEditBootItemBootString);
	iBootStringEdit->SetTextL(&iArloBootItem.iBootString);

	if (iArloBootItem.iDescription == EPOC_BOOT_DESCIPTION)
	{
		SetLineDimmedNow(ECtlNewOrEditBootItemDescription, ETrue);
		SetLineDimmedNow(ECtlNewOrEditBootItemKernel, ETrue);
		SetLineDimmedNow(ECtlNewOrEditBootItemInitrd, ETrue);
		SetLineDimmedNow(ECtlNewOrEditBootItemBootString, ETrue);
		iIsEpocBootItem = ETrue;
	}
}

void CNewOrEditBootItemDialog::SetSizeAndPositionL(const TSize& aSize)
{
//	TSize	size(400, 200);
	SetCornerAndSizeL(EHCenterVCenter, aSize);
}

void CNewOrEditBootItemDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch (aControlId)
	{
		case ECtlNewOrEditBootItemKernel:
			SelectFileL(&iArloBootItem.iKernelFileName, R_EDIT_OR_NEW_DIALOG_SELECT_KERNEL);
			SetKernelFileName();
			ikernelChoiceList->SetCurrentItem(0);
			break;
		case ECtlNewOrEditBootItemInitrd:
			if (iInitrdChoiceList->CurrentItem() == 1)
				SelectFileL(&iArloBootItem.iInitrdFileName, R_EDIT_OR_NEW_DIALOG_SELECT_INITRD);
			else if (iInitrdChoiceList->CurrentItem() == 2)
				iArloBootItem.iInitrdFileName = _L("");
			SetInitrdFileName();
			iInitrdChoiceList->SetCurrentItem(0);
			break;
	}
}

TBool CNewOrEditBootItemDialog::SelectFileL(TDes* aFileName, TInt aTitleId)
{
	TBool	isEmpty = (aFileName->Length() == 0);
	if (isEmpty)
		*aFileName = _L("c:\\");
	CEikFileOpenDialog*	fileOpenDialog = new(ELeave)CEikFileOpenDialog(aFileName, aTitleId);
	TBool ok = fileOpenDialog->ExecuteLD(R_EIK_DIALOG_FILE_OPEN) == EEikBidOk;
	if (!ok && isEmpty)
		*aFileName = _L("");
	return ok;
}


/************************************************
 *
 * About Dialog
 *
 ************************************************/


class CAboutDialog : public CEikDialog
{
public:
	CAboutDialog(){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
};

void CAboutDialog::PreLayoutDynInitL()
{
	TBuf<128>	text;
	TBuf<16>	versionText;

	versionText.Format(_L("%d.%d.%d"), ARLO_VERSION_MAJOR, ARLO_VERSION_MINOR, ARLO_VERSION_PATCH);

	HBufC*		versionFormat =	CEikonEnv::Static()->AllocReadResourceL(R_ABOUT_VERSION_TEXT);
	CleanupStack::PushL(versionFormat);

	const TDesC&	versionFormatDes = versionFormat->Des();
	text.Format(versionFormatDes, &versionText);

	CEikLabel*	label =(CEikLabel*)Control(ECtlAboutVersion);
	label->SetTextL(text);

	CleanupStack::PopAndDestroy(); // 	versionFormat;
}

void CAboutDialog::SetSizeAndPositionL(const TSize& aSize)
{
//	TSize	size(400, 200);
	SetCornerAndSizeL(EHCenterVCenter, aSize);
}


/************************************************
 *
 * Boot Item Dialog
 *
 ************************************************/

class CBootInfoDialog : public CEikDialog
{
public:
	CBootInfoDialog(const TArloBootItem& aArloBootItem) : iArloBootItem(aArloBootItem){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
	const TArloBootItem&		iArloBootItem;
};


void CBootInfoDialog::PreLayoutDynInitL()
{
	CEikLabel* label =(CEikLabel*)Control(ECtlBootInfoDescription);
	label->SetTextL(iArloBootItem.iDescription);

	label =(CEikLabel*)Control(ECtlBootInfoKernel);
	label->SetTextL(iArloBootItem.iKernelFileName);

	label =(CEikLabel*)Control(ECtlBootInfoInitrd);
	label->SetTextL(iArloBootItem.iInitrdFileName);

	label =(CEikLabel*)Control(ECtlBootInfoParameter);
	label->SetTextL(iArloBootItem.iBootString);
}


void CBootInfoDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize size = aSize;
	size.iWidth = 400;
	SetCornerAndSizeL(EHCenterVCenter, size);
}




/************************************************
 *
 * Boot List Dialog
 *
 ************************************************/

class CBootListDialog : public CEikDialog
{
public:
	~CBootListDialog();
	CBootListDialog(CArloConfig& aArloConfig, TInt aBootTimeout, TInt& aBootItemIndex) 
		: CEikDialog(), iArloConfig(aArloConfig), iBootItemIndex(aBootItemIndex),
		iBootTimeout(aBootTimeout) {}
protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();
	void HandleControlStateChangeL(TInt aControlId);
	void SetSizeAndPositionL(const TSize& aSize);
	void ShowBootItemDetailsL(TInt aIndex);
	TInt GetBootItemIndex();
    TBool OkToExitL(TInt aKeycode);
	TInt ItemIndex();

protected:	// tier support
	void CancelTimer();
	void StartTimer();
	void UpdateTimeoutLabel();
	void TimerTick();
	static TInt TimerCallBack(TAny* aAny);

	CPeriodic*		iPeriodic;
	CEikLabel*		iTimeoutLabel;

protected:
	CArloConfig&	iArloConfig;
	TInt&			iBootItemIndex;
	TInt			iBootTimeout;
	TBool			iBigScreen;
};


CBootListDialog::~CBootListDialog()
{
	CancelTimer();
}


void CBootListDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize size = aSize;
	if (iBigScreen)
		size.iWidth = 600;
	else if (size.iWidth < 400)
		size.iWidth = 400;

	SetCornerAndSizeL(EHCenterVCenter, size);
}


TInt CBootListDialog::ItemIndex()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlBootListListbox);
	return listbox->CurrentItemIndex();
}

void CBootListDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch(aControlId)
	{
		case ECtlBootListCmdButPanel:
		{
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlBootListCmdButBoot:
					TryExitL(EEikBidOk);
					break;
				case ECtlBootListCmdButCancel:
					TryExitL(EEikBidCancel);
					break;
				case ECtlBootListCmdButInfo:
					CancelTimer();
					ShowBootItemDetailsL(ItemIndex());
					break;
			}
			break;
		}
		case ECtlBootListListbox:
		{
			CancelTimer();
			break;
		}

	}
}

void CBootListDialog::ShowBootItemDetailsL(TInt aIndex)
{
	CEikDialog*	dialog = new CBootInfoDialog(iArloConfig.BootItem(aIndex));
	dialog->ExecuteLD(R_DIALOG_BOOT_INFO);
}


void CBootListDialog::PreLayoutDynInitL()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlBootListListbox);

		// determine wether we need big width
	TInt count = iArloConfig.BootItems();
	for (TInt i=0 ; i<count ; i++)
		if (iArloConfig.BootItem(i).iDescription.Length() > 35)
			iBigScreen = ETrue;

	CEikScrollBarFrame* scrollbarFrame = listbox->CreateScrollBarFrameL();
	// set horizontal and vertical scroll bars to appear when required
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);

	if (iBootTimeout >= 0)
	{
		iTimeoutLabel = (CEikLabel*)Control(ECtlBootListBootTimeout);
		iBootTimeout = iArloConfig.Timeout();
		UpdateTimeoutLabel();
		StartTimer();
	}
	else
	{
		DeleteLine(ECtlBootListBootTimeout);
	}
}

void CBootListDialog::PostLayoutDynInitL()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlBootListListbox);

	CDesCArray* listboxArray=((CDesCArray*)listbox->Model()->ItemTextArray());

	TInt count = iArloConfig.BootItems();
	for (TInt i=0 ; i<count ; i++)
	{
		listboxArray->AppendL(iArloConfig.BootItem(i).iDescription);
		if (iArloConfig.BootItem(i).iDescription.Length() > 35)
			iBigScreen = ETrue;
	}
	listbox->HandleItemAdditionL();

	TInt highlight = iArloConfig.DefaultBootItem();
	if ((highlight < 0) || (highlight>=count))
		highlight = 0;

	listbox->SetCurrentItemIndex(highlight);

	CEikScrollBarFrame* scrollbarFrame = listbox->CreateScrollBarFrameL();
	// set horizontal and vertical scroll bars to appear when required
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);
}


void CBootListDialog::UpdateTimeoutLabel()
{
	TBuf<64>	text;
	text.Format(_L("%d seconds"), iBootTimeout);
	iTimeoutLabel->SetTextL(text);
	iTimeoutLabel->DrawNow();
}

TInt CBootListDialog::TimerCallBack(TAny* aAny)
{
	CBootListDialog*	bootListDialog = (CBootListDialog*) aAny;
	bootListDialog->TimerTick();
	return ETrue;
}

void CBootListDialog::TimerTick()
{
	iBootTimeout--;
	UpdateTimeoutLabel();
	if (iBootTimeout == 0)
		TryExitL(EEikBidOk);
}


void CBootListDialog::StartTimer()
{
	iPeriodic = CPeriodic::NewL(0);
	iPeriodic->Start(1 * 1000 * 1000, 1 * 1000 * 1000, TCallBack(&CBootListDialog::TimerCallBack, this));
}

void CBootListDialog::CancelTimer()
{
	if (iTimeoutLabel)
	{
		iTimeoutLabel = NULL;

		DeleteLine(ECtlBootListBootTimeout);

		delete iPeriodic;
		iPeriodic = NULL;

		DrawNow();
	}
}


TInt CBootListDialog::GetBootItemIndex()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlBootListListbox);
	return listbox->CurrentItemIndex();
}

TBool CBootListDialog::OkToExitL(TInt /*aKeycode*/)
{
	CancelTimer();
	iBootItemIndex = GetBootItemIndex();
    return(ETrue);
}





/************************************************
 *
 * Options boot List Dialog
 *
 ************************************************/

class COptionsBootListDialog : public CEikDialog
{
public:
	~COptionsBootListDialog();
	COptionsBootListDialog(CArloConfig& aArloConfig) 
		: CEikDialog(), iArloConfig(aArloConfig){}
protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();
	void HandleControlStateChangeL(TInt aControlId);
	void SetSizeAndPositionL(const TSize& aSize);

	void EditBootItemDetailsL(TInt aIndex);
	void NewBootItemL();
	void DeleteBootItem(TInt aIndex);

	TInt GetBootItemIndex();
    TBool OkToExitL(TInt aKeycode);
	TInt ItemIndex();

	void PopulateListBox(TInt aHighlight);
	void MoveUp(TInt aIndex);
	void MoveDown(TInt aIndex);

protected:
	CArloConfig&		iArloConfig;

	CEikTextListBox*	iListbox;
};


COptionsBootListDialog::~COptionsBootListDialog()
{
}


void COptionsBootListDialog::SetSizeAndPositionL(const TSize& aSize)
{
	TSize size = aSize;
	if (size.iWidth < 400)
		size.iWidth = 400;

	SetCornerAndSizeL(EHCenterVCenter, size);
}


TInt COptionsBootListDialog::ItemIndex()
{
	return iListbox->CurrentItemIndex();
}


void COptionsBootListDialog::PopulateListBox(TInt aHighlight)
{
	iListbox->Reset();
	CDesCArray* listboxArray=((CDesCArray*)iListbox->Model()->ItemTextArray());
	listboxArray->Reset();
	TInt count = iArloConfig.BootItems();
	for (TInt i=0 ; i<count ; i++)
		listboxArray->AppendL(iArloConfig.BootItem(i).iDescription);
	iListbox->HandleItemAdditionL();

	if ((aHighlight < 0) || (aHighlight>=count)) 
		aHighlight = 0;
	iListbox->SetCurrentItemIndex(aHighlight);
	iListbox->DrawNow();
}

void COptionsBootListDialog::MoveUp(TInt aIndex)
{
	if (iArloConfig.SwapItems(aIndex, aIndex-1))
		PopulateListBox(aIndex-1);
}

void COptionsBootListDialog::MoveDown(TInt aIndex)
{
	if (iArloConfig.SwapItems(aIndex, aIndex+1))
		PopulateListBox(aIndex+1);
}

void COptionsBootListDialog::DeleteBootItem(TInt aIndex)
{
	TInt highlight = (aIndex == iArloConfig.BootItems()-1) ? aIndex - 1 : aIndex;
	if (iArloConfig.BootItem(aIndex).iDescription != EPOC_BOOT_DESCIPTION)
	{
		iArloConfig.DeleteBootItem(aIndex);
		PopulateListBox(highlight);
	}
	else
	{
		User::InfoPrint(_L("This item cannot be deleted"));
	}
}

void COptionsBootListDialog::HandleControlStateChangeL(TInt aControlId)
{
	switch(aControlId)
	{
		case ECtlOptionBootListCmdButPanel1:
		{
			CEikDialogToolBar* toolBar = STATIC_CAST(CEikDialogToolBar*, Control(ECtlOptionBootListCmdButPanel1));
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			if (ctrlId == ECtlOptionsBootListToolBarUpDown)
			{
				const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, toolBar->ControlById(ECtlOptionsBootListToolBarUpDown))->StateChangeControlId();

				switch (ctrlId)
				{
					case ECtlOptionsBootListCmdButUp:
						MoveUp(ItemIndex());
						break;
					case ECtlOptionsBootListCmdButDown:
						MoveDown(ItemIndex());
						break;
				}
				break;
			}
			break;
		}
		case ECtlOptionBootListCmdButPanel2:
		{
			const TInt ctrlId=STATIC_CAST(CEikDialogToolBar*, Control(aControlId))->StateChangeControlId();
			switch (ctrlId)
			{
				case ECtlOptionsBootListCmdButOk:
					TryExitL(EEikBidOk);
					break;
				case ECtlOptionsBootListCmdButCancel:
					TryExitL(EEikBidCancel);
					break;
				case ECtlOptionsBootListCmdButEdit:
					EditBootItemDetailsL(ItemIndex());
					break;
				case ECtlOptionsBootListCmdButDelete:
					DeleteBootItem(ItemIndex());
					break;
				case ECtlOptionsBootListCmdButNew:
					NewBootItemL();
					break;
			}
			break;
		}
	}
}

void COptionsBootListDialog::EditBootItemDetailsL(TInt aIndex)
{
	TArloBootItem	arloBootItem = iArloConfig.BootItem(aIndex);
	CEikDialog*	dialog = new CNewOrEditBootItemDialog(iArloConfig, arloBootItem, EFalse);
	dialog->PrepareLC(R_DIALOG_NEW_OR_EDIT_BOOT_ITEM);
	dialog->SetTitleL(R_EDIT_OR_NEW_DIALOG_EDIT_TITLE);
	if (dialog->RunLD() == EEikBidOk)
	{
		iArloConfig.SetBootItem(aIndex, arloBootItem);
		PopulateListBox(aIndex);
	}
}


void COptionsBootListDialog::NewBootItemL()
{
	TArloBootItem*	arloBootItem = iArloConfig.CreateNewBootItemLC();
	CEikDialog*	dialog = new CNewOrEditBootItemDialog(iArloConfig, *arloBootItem, ETrue);
	dialog->PrepareLC(R_DIALOG_NEW_OR_EDIT_BOOT_ITEM);
	dialog->SetTitleL(R_EDIT_OR_NEW_DIALOG_NEW_TITLE);
	if (dialog->RunLD() == EEikBidOk)
	{
		iArloConfig.AddBootItemL(arloBootItem);
		PopulateListBox(iArloConfig.BootItems() - 1);
	}
	CleanupStack::PopAndDestroy();
}

void COptionsBootListDialog::PostLayoutDynInitL()
{
	PopulateListBox(iArloConfig.DefaultBootItem());
	iListbox->SetFocus(ETrue, EDrawNow);
}

void COptionsBootListDialog::PreLayoutDynInitL()
{
	// layout up and down buttons vertically
	CEikDialogToolBar* toolBar = STATIC_CAST(CEikDialogToolBar*, Control(ECtlOptionBootListCmdButPanel1));
	CEikDialogToolBar* upDownToolBar = STATIC_CAST(CEikDialogToolBar*, toolBar->ControlById(ECtlOptionsBootListToolBarUpDown));
	upDownToolBar->SetControlLayout(CEikControlGroup::EFromTopLeft, CEikControlGroup::ELayVertically);

	iListbox=STATIC_CAST(CEikTextListBox*, toolBar->ControlById(ECtlOptionsBootListListbox));

	CEikScrollBarFrame* scrollbarFrame = iListbox->CreateScrollBarFrameL();
	// set horizontal and vertical scroll bars to appear when required
	scrollbarFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto,
		CEikScrollBarFrame::EAuto);
}



TInt COptionsBootListDialog::GetBootItemIndex()
{
	CEikTextListBox* listbox=(CEikTextListBox*)Control(ECtlOptionsBootListListbox);
	return listbox->CurrentItemIndex();
}

TBool COptionsBootListDialog::OkToExitL(TInt /*aKeycode*/)
{
    return(ETrue);
}





/************************************************
 *
 * Application
 *
 ************************************************/

CApaDocument* CArloApplication::CreateDocumentL()
{
	CArloDocument*	doc = new (ELeave) CArloDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}


/************************************************
 *
 * Document
 *
 ************************************************/

CArloDocument::CArloDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}


CArloDocument::~CArloDocument()
{
}

void CArloDocument::ConstructL()
{
}


CEikAppUi* CArloDocument::CreateAppUiL()
{
	return new(ELeave) CArloAppUi;
}


/************************************************
 *
 * App UI
 *
 ************************************************/

CArloAppUi::~CArloAppUi()
{
	CancelStartupTimer();
	delete iAppView;
}


void CArloAppUi::ConstructL()
{
	BaseConstructL();
	iAppView=new(ELeave) CArloAppView;
	iAppView->ConstructL(ClientRect());

	iArloEngine = CArloEngine::NewL(&CCoeEnv::Static()->WsSession(), &CCoeEnv::Static()->FsSession());

	iBootTimeout = iArloEngine->ArloConfig().Timeout();

	// sothat the background logo is visible for a short while
	StartStartupTimer();
}

void CArloAppUi::StartStartupTimer()
{
	iPeriodic = CPeriodic::NewL(0);
	iPeriodic->Start(1 * 1000 * 1000, 0, TCallBack(&CArloAppUi::TimerCallBack, this));
}

void CArloAppUi::CancelStartupTimer()
{
	delete iPeriodic;
	iPeriodic = NULL;
}

void CArloAppUi::StartupTimerFired()
{
	CancelStartupTimer();
	DisplayBootListDialogL();
}

TInt CArloAppUi::TimerCallBack(TAny* aAny)
{
	CArloAppUi* eLoadLinAppUi = (CArloAppUi*) aAny;
	eLoadLinAppUi->StartupTimerFired();
	return ETrue;
}



TBool CArloAppUi::IsColorDevice()
{
	TDisplayMode	displayMode = CEikonEnv::Static()->ScreenDevice()->DisplayMode();

	return (displayMode >= EColor16);
}


void CArloAppUi::HandleCommandL(TInt aCommand)
{
	CancelStartupTimer();

	switch (aCommand)
	{
		case EEikCmdExit: 
			Exit();
			break;

		case EEikCmdBootList:
			DisplayBootListDialogL();
			break;

		case EEikCmdOptionsBootList:
			DisplayOptionsBootListDialogL();
			break;

		case EEikCmdAbout:
			DisplayAboutDialogL();
			break;

		case EEikCmdOptionsArlo:
			DisplayArloOptionsDialogL();
			break;

		case EEikCmdOptionsSave:
			SaveConfigFile();
			break;

		case EEikCmdArloInfo:
			DisplayArloInfoL();
			break;
	}
}


void CArloAppUi::DisplayOptionsBootListDialogL()
{
	CArloConfig* arloConfig = iArloEngine->ArloConfig().CloneLC();

	CEikDialog*	dialog = new(ELeave) COptionsBootListDialog(*arloConfig);
	if (dialog->ExecuteLD(R_DIALOG_OPTIONS_BOOT_LIST) == EEikBidOk)
		iArloEngine->SetConfig(arloConfig);
	else
		CleanupStack::PopAndDestroy();
}


void CArloAppUi::SaveConfigFile()
{
	iArloEngine->SaveConfig();
}


void CArloAppUi::DisplayBootListDialogL()
{
	TInt	bootItemIndex;
	CEikDialog*	dialog = new (ELeave) CBootListDialog(iArloEngine->ArloConfig(), iBootTimeout, bootItemIndex);
	if (dialog->ExecuteLD(R_DIALOG_BOOT_LIST))
	{
		const TArloBootItem& bootItem = iArloEngine->ArloConfig().BootItem(bootItemIndex);
		iArloEngine->BootLinux(bootItem);
	}
	else
	{
		iArloEngine->StartEpocIfRunningAsBootLoader();
	}
	iBootTimeout = -1;
}


void CArloAppUi::DisplayArloInfoL()
{
	CEikDialog*	dialog = new(ELeave) CArloInfoDialog(iArloEngine->Arlo().ArloGetProcId());
	dialog->ExecuteLD(R_DIALOG_ARLO_INFO);
}

void CArloAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new(ELeave) CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}

void CArloAppUi::DisplayArloOptionsDialogL()
{
	CArloConfig* arloConfig = iArloEngine->ArloConfig().CloneLC();

	CEikDialog*	dialog = new CArloOptionsDialog(*arloConfig);
	if (dialog->ExecuteLD(R_DIALOG_ARLO_OPTIONS) == EEikBidOk)
		iArloEngine->SetConfig(arloConfig);
	else
		CleanupStack::PopAndDestroy();
}


/************************************************
 *
 * App view
 *
 ************************************************/

void CArloAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRectL(aRect);
	ActivateL();

		/*
		 * Load background image; either grayscale or color depending on the device
		 * we're running on.
		 */
	TBool	isColorDevice = CArloAppUi::Static()->IsColorDevice();
	TInt	image = isColorDevice ? EMbmArloTitle256 : EMbmArloTitle256;
	TRAPD(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(_L("c:\\system\\apps\\arlo\\arlo.mbm"), image));
	if (err)
	{
		TRAP(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(_L("d:\\system\\apps\\arlo\\arlo.mbm"), image));
	}

	TInt	image1 = isColorDevice ? EMbmArloLogo256 : EMbmArloLogo256;
	TRAP(err, iBgImage1 = CEikonEnv::Static()->CreateBitmapL(_L("c:\\system\\apps\\arlo\\arlo.mbm"), image1));
	if (err)
	{
		TRAP(err, iBgImage1 = CEikonEnv::Static()->CreateBitmapL(_L("d:\\system\\apps\\arlo\\arlo.mbm"), image1));
	}
	if (iBgImage && iBgImage1)
	{
		TInt	gap = isColorDevice ? 20 : 2;
		iImagePosition.iX = (aRect.Size().iWidth - iBgImage->SizeInPixels().iWidth) / 2;
		iImagePosition.iY = (aRect.Size().iHeight - (iBgImage->SizeInPixels().iHeight + iBgImage1->SizeInPixels().iHeight + gap)) / 2;

		iImagePosition1.iX = (aRect.Size().iWidth - iBgImage1->SizeInPixels().iWidth) / 2;
		iImagePosition1.iY = iImagePosition.iY + iBgImage->SizeInPixels().iHeight + gap;
	}
}

CArloAppView::~CArloAppView()
{
	delete iBgImage;
	delete iBgImage1;
}


void CArloAppView::Draw(const TRect& /*aRect*/) const
{
	CWindowGc& gc = SystemGc();

	if (iBgImage && iBgImage1)
	{
		gc.DrawBitmap(iImagePosition, iBgImage);
		gc.DrawBitmap(iImagePosition1, iBgImage1);
	}
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CArloApplication;
}



TUid CArloApplication::AppDllUid() const
{
	return KUidArloApp;
}
