/*******************************************************************
 *
 *	File:		CArloEngine.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <e32cons.h>
#include <f32file.h>
#include <e32svr.h>
#include <w32std.h>

#include "CArloEngine.h"
#include "CArlo.h"
#include "RArlo.h"
#include "CArloConfig.h"

#ifdef __WINS__

#define EKERN_EXE_FILENAME		_L("c:\\test\\ekern.exe")
#define ECOMM_LDD_FILENAME		_L("c:\\test\\ecomm.ldd")
#define ARLO_LDD_FILENAME_ON_C	_L("c:\\test\\arloc.ldd")
#define ARLO_LDD_FILENAME_ON_D	_L("c:\\test\\arlod.ldd")

#else

#define EKERN_EXE_FILENAME		_L("z:\\system\\libs\\ekern.exe")
#define ECOMM_LDD_FILENAME		_L("z:\\system\\libs\\ecomm.ldd")
#define ARLO_LDD_FILENAME_ON_C	_L("c:\\system\\libs\\arlo.ldd")
#define ARLO_LDD_FILENAME_ON_D	_L("d:\\system\\libs\\arlo.ldd")

#endif


static TBool RunsAsSystemBootLoaderL(RWsSession& aWsSession)
{
	TInt			id=0;
	TInt			count = 0;

		/*
		 * We count the number of named window groups.
		 * If we run as a boot loader, the count is 1, otherwise there are about 10 of them
		 */
	while (id >= 0)
	{
		id = aWsSession.FindWindowGroupIdentifier(id , _L("*"));
		if (id > 0)
		{
			TThreadId threadId;
			if (aWsSession.GetWindowGroupClientThreadId(id , threadId) == KErrNone)
			{
				RThread	thread;
				if (thread.Open(threadId) == KErrNone)
				{
					thread.Close();
					count++;
//					iConsole.Printf(_L("Threads: thread=%S, id=%d\n"), &name, id);
				}
			}
		}
	}
	return (count == 1);
}



static TInt GetFileUids(const TDesC& aFileName, TUint& aUid0, TUint& aUid1, TUint& aUid2)
{
	RFs				fs;
	
	TInt err= fs.Connect();
	if (!err)
	{
		RFile		file;
		err = file.Open(fs, aFileName, 0);
		if (!err)
		{
			TBuf8<12>	uids;
			err = file.Read(uids);
			if (!err)
			{
				TUint*	p = (TUint*) uids.Ptr();
				aUid0 = p[0];
				aUid1 = p[1];
				aUid2 = p[2];
			}
			file.Close();
		}
		fs.Close();
	}
	return err;
}


/*
 * What we need to patch is:
 * 1) The first 2 UIDs in the arlo image file. For that we simply copy the 2 UIDs from
 * ecomm.ldd
 * 2) Arlo links against EKERN.EXE. Somewhere in the arlo image it says:
 * "EKERN[1000008b].EXEEUSER[100000c1].DLL". This is the section used by the loader
 * to determine against which .dlls this image has linked. The UID that is specified
 * between square brackets is the 3rd UID of that according binary image. In our case
 * the EKERN UIDs are unique per platform.
 */
TBool PatchDataIfNeeded(TDes8& aData, TUint aEkernUid, TUint aEcommUid0, TUint aEcommUid1)
{
	TBool	isChanged = EFalse;
	TUint*	uid = (TUint*) aData.Ptr();
	if (uid[0] != aEcommUid0)
	{
		uid[0] = aEcommUid0;
		isChanged = ETrue;
	}
	if (uid[1] != aEcommUid1)
	{
		uid[1] = aEcommUid1;
		isChanged = ETrue;
	}

	TInt pos = aData.Find(_L("EKERN["));
	if (pos != KErrNotFound)
	{
		pos += 6 + 7;
		TUint8*	data = (TUint8*) aData.Ptr() + pos;
		for (TInt i=0 ; i<8 ; i++)
		{
			TUint	c = (aEkernUid & 0xf);
			c += (c<10) ? '0' : ('a' - 10);
			if (*data != (TUint8) c)
			{
				*data = (TUint8) c;
				isChanged = ETrue;
			}
			data--;
			aEkernUid >>= 4;
		}
	}
	return isChanged;
}

static TBool PatchArloImageIfNeededL(const TDesC& aFilename, TUint aEkernUid, TUint aEcommUid0, TUint aEcommUid1)
{
	TBool		isChanged = EFalse;
	
	RFs		fsSession;
	User::LeaveIfError(fsSession.Connect());
	CleanupClosePushL(fsSession);

	RFile file;
	User::LeaveIfError(file.Open(fsSession, aFilename, EFileWrite));
	CleanupClosePushL(file);

	TInt fileSize;
	User::LeaveIfError(file.Size(fileSize));
	HBufC*	buf = HBufC::NewLC(fileSize);

	TBuf<32>	info;
	info.Format(_L("File size=%d"), fileSize);

	TPtr	ptr = buf->Des();
	User::LeaveIfError(file.Read(ptr));

	isChanged = PatchDataIfNeeded(ptr, aEkernUid, aEcommUid0, aEcommUid1);
	if (isChanged)
	{
			/*
			 * Unbelievable!
			 * file.Write(0, *buf) works fine on the Series5mx + the Series 7 but fails
			 * to work correctly on the Series 5. What gives !?
			 */
		User::LeaveIfError(file.Write(0, ptr));
	}

	CleanupStack::PopAndDestroy(3);	// buf, file, fsSession

	return isChanged;
}


static void PatchArloDriverIfNeededL()
{
	TUint	ekernUid, ecommUid0, ecommUid1, dummy;

	User::LeaveIfError(GetFileUids(EKERN_EXE_FILENAME, dummy, dummy, ekernUid));
	User::LeaveIfError(GetFileUids(ECOMM_LDD_FILENAME, ecommUid0, ecommUid1, dummy));

	TBool	isChanged = EFalse;
	TRAPD(err, isChanged = PatchArloImageIfNeededL(ARLO_LDD_FILENAME_ON_C, ekernUid, ecommUid0, ecommUid1));
	if (!err && isChanged)
	{
		User::InfoPrint(_L("Arlo driver successfully patched for this platform\n"));
		User::After(2 * 1000 * 1000);
	}

	TRAPD(err1, isChanged = PatchArloImageIfNeededL(ARLO_LDD_FILENAME_ON_D, ekernUid, ecommUid0, ecommUid1));
	if (!err1 && isChanged)
	{
		User::InfoPrint(_L("Arlo driver successfully patched for this platform\n"));
		User::After(2 * 1000 * 1000);
	}
}

static void PatchArloDriverIfNeeded()
{
	TRAPD(err, PatchArloDriverIfNeededL());
}


static TInt LoadArloDriver(RArLo& aArlo)
{
//	aConsole->Printf(_L("Loading Arlo device driver\n"));

	TInt err = User::LoadLogicalDevice(KArloDriverName);
	if ((err!=KErrNone) && (err!=KErrAlreadyExists))
	{
//		aConsole->Printf(_L("Error %d while loading logical device\n"), err);
//		DumpUidInfo(aConsole);
	}
	else
	{
		if (err == KErrAlreadyExists)
		{
//			aConsole->Printf(_L("  (driver was already loaded)\n"));
		}
//		aConsole->Printf(_L("Opening the channel\n"));
		err = aArlo.Open(aArlo.VersionRequired());
		if (err) 
		{
//			aConsole->Printf(_L("Error %d while opening channel to Arlo device driver\n"), err);
			User::FreeLogicalDevice(KArloDriverName);
		}
	}
//	DumpUidInfo(aConsole);

	return err;
}


/*********************************************
 *
 * Arlo engine
 *
 *********************************************/

CArloEngine::~CArloEngine()
{
	if (iOwnsWsSession && iWsSession)
	{
		if (iWsSession->Handle() != -1)
			iWsSession->Close();
		delete iWsSession;
	}
}


CArloEngine::CArloEngine(RWsSession* aWsSession, RFs* aFs) : iWsSession(aWsSession), iFs(aFs)
{
}

CArloEngine* CArloEngine::NewL(RWsSession* aWsSession, RFs* aFs)
{
	CArloEngine*	self = new(ELeave)CArloEngine(aWsSession, aFs);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}


void CArloEngine::ConstructL()
{
	if (!iWsSession)
	{
		iWsSession = new(ELeave)RWsSession;
#ifndef __WINS__

			/*
			 * Under WINS there is no Window Server. That's fine, because the
			 * info on the video setup (size & mode etc) is only needed fo target.
			 */
		User::LeaveIfError(iWsSession->Connect());
#endif
		iOwnsWsSession = ETrue;
	}

	if (!iFs)
	{
		iFs = new(ELeave)RFs;
		iOwnsFs = ETrue;
	}

	PatchArloDriverIfNeeded();
	TInt err = LoadArloDriver(iArloDevice);
	if (err)
	{
		TBuf<64>	errMsg;
		errMsg.Format(_L("Could not load driver, err=%d"), err);
		User::InfoPrint(errMsg);
		User::After(3 * 1000 * 1000);
	}

	iArloConfig = CArloConfig::ReadConfigFileLC();
	CleanupStack::Pop();			// iArloConfig

	ResetArloL();

	iRunsAsSystemBootLoader = EFalse;
#ifndef __WINS__
	TRAP(err, (iRunsAsSystemBootLoader = RunsAsSystemBootLoaderL(*iWsSession)));
#endif
}

void CArloEngine::ResetArloL()
{
	delete iArlo;
	iArlo = NULL;

	iArlo = ArloFactory::CreateArloLC(iArloDevice, iArloConfig->MachineName(), *iFs, *iWsSession);
	CleanupStack::Pop();		// iArlo
}


void CArloEngine::ConfigArlo(const TArloBootItem& aArloBootItem)
{
	iArlo->UseBoot2nd(iArloConfig->UseBoot2nd());	// must be first statement
	iArlo->SetBootString((TAny *) aArloBootItem.iBootString.Ptr(), aArloBootItem.iBootString.Length());

		/*
		 * Override (if supported by this machine) some parameters if specified in the config file
		 */
	boot1st_param_t*	boot1st_param = iArlo->Boot1stParam();
	if (boot1st_param)
	{
		if (aArloBootItem.iKernelLoadAdr)
			boot1st_param->kernel_reloc_dst = (unsigned*) aArloBootItem.iKernelLoadAdr;
		if (aArloBootItem.iInitrdLoadAdr)
			boot1st_param->initrd_reloc_dst = (unsigned*) aArloBootItem.iInitrdLoadAdr;
		if (aArloBootItem.iHasColoursSet)
		{
			boot1st_param->video_fg_colour = iArlo->NearestColourIndex(TRgb((TUint32)aArloBootItem.iForegroundColour));
			boot1st_param->video_bg_colour = iArlo->NearestColourIndex(TRgb((TUint32)aArloBootItem.iBackgroundColour));
		}
		if (aArloBootItem.iArchNumber != 0)
		{
			boot1st_param->arm_arch_no = aArloBootItem.iArchNumber;
		}
	}
}

void CArloEngine::LoadKernelImageL(const TArloBootItem& aArloBootItem)
{
	TUint		fileSize;
	TUint32		loadAdr;
	TUint		blocks;

	iArlo->LoadFileL(aArloBootItem.iKernelFileName, fileSize, loadAdr, blocks);
}

void CArloEngine::LoadInitrdImageL(const TArloBootItem& aArloBootItem)
{
	TUint		fileSize;
	TUint32		loadAdr;
	TUint		blocks;

	if (aArloBootItem.iInitrdFileName.Length() > 0)
		iArlo->LoadFileL(aArloBootItem.iInitrdFileName, fileSize, loadAdr, blocks);
}


void CArloEngine::StartEpoc()
{
	if (iRunsAsSystemBootLoader)
	{
		RProcess	me;
		me.Terminate(KErrNone);
	}
	else
		User::InfoPrint(_L("EPOC is already running"));
}


TBool CArloEngine::BootLinux(const TArloBootItem& aBootItem)
{
	TInt err;
	TBuf<64>	msg;

	if (aBootItem.iDescription == EPOC_BOOT_DESCIPTION)
	{
		StartEpoc();
		return ETrue;
	}

	ConfigArlo(aBootItem);
	User::InfoPrint(aBootItem.iKernelFileName);
	TRAP(err, LoadKernelImageL(aBootItem));
	if (!err)
	{
		if (aBootItem.iInitrdFileName.Length())
		{
			User::InfoPrint(aBootItem.iInitrdFileName);
			TRAP(err, LoadInitrdImageL(aBootItem));
			if (err)
			{
				User::After(2 * 1000 * 1000);
				msg.Format(_L("Could not load ramdisk image (err=%d)"), err);
				User::InfoPrint(msg);
			}
		}
		if (!err)
		{
			iArlo->ArloBoot();
		}
	}
	else
	{
		User::After(2 * 1000 * 1000);
		msg.Format(_L("Could not load kernel image (err=%d)"), err);
		User::InfoPrint(msg);
	}

	// reset ArloState. If we can't we have to panic!
	TRAP(err, ResetArloL())

	if (err)
	{
		msg.Format(_L("Fatal error recovering failed boot (err=%d)"), err);
		User::InfoPrint(msg);
		User::After(3 * 1000 * 1000);
		User::Panic(_L("Arlo"), err);
	}
	return EFalse;
}


TBool CArloEngine::SaveConfig()
{
		/*
		 * If SaveConfig returns ETrue, we've taken ownership of the aArloConfig 
		 * which should be on the cleanup stack. If SaveConfig returns EFalse,
		 * aArloConfig will still be on the cleanup stack.
		 */
	TBool	saved = EFalse;
	if (iArloConfig->Save())
	{
		TBuf<128>	text;
		text.Format(_L("Saved %S"), &iArloConfig->UsedConfigFile());
		User::InfoPrint(text);
		saved = ETrue;
	}
	else
	{
		User::InfoPrint(_L("Data could not be saved"));
	}
	return saved;
}


void CArloEngine::SetConfig(CArloConfig* aArloConfig)
{
	delete iArloConfig;
	iArloConfig = aArloConfig;
	CleanupStack::Pop();
}

void CArloEngine::StartEpocIfRunningAsBootLoader()
{
	if (iRunsAsSystemBootLoader)
		StartEpoc();
}
